@@ -26,6 +26,8 @@ struct checksums;
 
 extern const char * const changes_checksum_names[];
 extern const char * const source_checksum_names[];
+extern const char * const release_checksum_names[];
+extern const struct constant *hashnames;
 
 struct hashes {
 	struct hash_data {
@@ -78,6 +80,11 @@ retvalue checksums_replace(const char *filename, const char *, size_t len, /*@ou
  * RET_NOTHING means file does not exist, RET_ERROR_WRONG_MD5 means wrong size */
 retvalue checksums_cheaptest(const char *fullfilename, const struct checksums *, bool);
 
+/* check if filename has specified checksums, if not return RET_ERROR_WRONG_MD5,
+ * if it has, and checksums_p put the improved checksum there (*checksums_p should
+ * either be NULL or checksums */
+retvalue checksums_test(const char *, const struct checksums *, /*@null@*/struct checksums **);
+
 /* check if checksum of filekey in database and checksum of actual file, set improve if some new has is in the last */
 bool checksums_check(const struct checksums *, const struct checksums *, /*@out@*/bool *improves);
 
@@ -87,7 +94,7 @@ bool checksums_iscomplete(const struct checksums *);
 /* Collect missing checksums (if all are there always RET_OK without checking).
  * if the file is not there, return RET_NOTHING,
  * if it is but not matches, return RET_ERROR_WRONG_MD5 */
-retvalue checksums_complete(struct checksums **, const char *fullfilename, /*@null@*/ bool improvedhashes[cs_hashCOUNT]);
+retvalue checksums_complete(struct checksums **, const char *fullfilename);
 
 void checksums_printdifferences(FILE *,const struct checksums *expected, const struct checksums *got);
 
@@ -107,6 +114,9 @@ void checksumsarray_resetunsupported(const struct checksumsarray *, bool[cs_hash
 
 retvalue hashline_parse(const char *filenametoshow, const char *line, enum checksumtype cs, /*@out@*/const char **basename_p, /*@out@*/struct hash_data *data_p, /*@out@*/struct hash_data *size_p);
 
+struct configiterator;
+retvalue checksums_parseignorelist(bool ignorelist[cs_hashCOUNT], const char *, struct configiterator *);
+
 #ifdef CHECKSUMS_CONTEXT
 #ifndef MD5_H
 #include "md5.h"
