@@ -4,6 +4,9 @@
 #ifndef REPREPRO_STRLIST_H
 #include "strlist.h"
 #endif
+#ifndef REPREPRO_ATOMS_H
+#include "atoms.h"
+#endif
 
 struct target;
 struct logger;
@@ -12,6 +15,8 @@ enum log_action { LOG_PACKAGE_ADD, LOG_PACKAGE_REPLACE, LOG_PACKAGE_REMOVE};
 
 /* file causing the current logger_log* run */
 extern /*@null@*/ const char *causingfile;
+/* command causing the current logger_log* run */
+extern command_t causingcommand_atom;
 
 retvalue logger_init(struct configiterator *, /*@out@*/struct logger **);
 void logger_free(/*@only@*/struct logger *);
