@@ -46,8 +46,6 @@
 #include "distribution.h"
 #include "release.h"
 
-extern int verbose;
-
 #define INPUT_BUFFER_SIZE 1024
 #define GZBUFSIZE 40960
 #define BZBUFSIZE 40960
@@ -374,7 +372,7 @@ static retvalue release_usecached(struct release *release,
 				r = RET_ERROR_OOM;
 			else
 				r = checksums_complete(&checksums[ic],
-					fullfilename, NULL);
+					fullfilename);
 			if( r == RET_ERROR_WRONG_MD5 ) {
 				fprintf(stderr,
 "WARNING: '%s' is different from recorded checksums.\n"
@@ -907,9 +905,12 @@ static retvalue finishbz(struct filetorelease *f) {
 		bzret = BZ2_bzCompress(&f->bzstream,BZ_FINISH);
 		f->bz_waiting_bytes = BZBUFSIZE - f->bzstream.avail_out;
 
-		if( bzret == BZ_RUN_OK || bzret == BZ_STREAM_END ) {
+		/* BZ_RUN_OK most likely is not possible here, but BZ_FINISH_OK
+		 * is returned when it cannot be finished in one step.
+		 * but better safe then sorry... */
+		if( (bzret == BZ_RUN_OK || bzret == BZ_FINISH_OK || bzret == BZ_STREAM_END)
+		    && f->bz_waiting_bytes > 0 ) {
 			retvalue r;
-			assert( f->bz_waiting_bytes != 0 );
 			r = writetofile(&f->f[ic_bzip2],
 					(const unsigned char*)f->bzoutputbuffer,
 					f->bz_waiting_bytes);
@@ -918,7 +919,7 @@ static retvalue finishbz(struct filetorelease *f) {
 				return r;
 			f->bz_waiting_bytes = 0;
 		}
-	} while( bzret == BZ_RUN_OK );
+	} while( bzret == BZ_RUN_OK || bzret == BZ_FINISH_OK );
 
 	if( bzret != BZ_STREAM_END ) {
 		fprintf(stderr,"Error from bzlib's bzCompress: "
@@ -1096,10 +1097,12 @@ retvalue release_directorydescription(struct release *release, const struct dist
 
 	release_writeheader("Archive",distribution->suite);
 	release_writeheader("Version",distribution->version);
-	release_writeheader("Component",target->component);
+	release_writeheader("Component",
+			atoms_components[target->component_atom]);
 	release_writeheader("Origin",distribution->origin);
 	release_writeheader("Label",distribution->label);
-	release_writeheader("Architecture",target->architecture);
+	release_writeheader("Architecture",
+			atoms_architectures[target->architecture_atom]);
 	release_writeheader("NotAutomatic",distribution->notautomatic);
 	release_writeheader("Description",distribution->description);
 #undef release_writeheader
@@ -1143,7 +1146,7 @@ static retvalue storechecksums(struct release *release) {
 retvalue release_prepare(struct release *release, struct distribution *distribution, bool onlyifneeded) {
 	size_t s;
 	retvalue r;
-	char buffer[100];
+	char buffer[100], untilbuffer[100];
 	time_t t;
 	struct tm *gmt;
 	struct release_entry *file;
@@ -1162,11 +1165,23 @@ retvalue release_prepare(struct release *release, struct distribution *distribut
 	if( gmt == NULL ) {
 		return RET_ERROR_OOM;
 	}
-	s=strftime(buffer,99,"%a, %d %b %Y %H:%M:%S +0000",gmt);
+	s=strftime(buffer,99,"%a, %d %b %Y %H:%M:%S UTC",gmt);
 	if( s == 0 || s >= 99) {
 		fprintf(stderr,"strftime is doing strange things...\n");
 		return RET_ERROR;
 	}
+	if( distribution->validfor > 0 ) {
+		t += distribution->validfor;
+		gmt = gmtime(&t);
+		if( gmt == NULL ) {
+			return RET_ERROR_OOM;
+		}
+		s=strftime(untilbuffer,99,"%a, %d %b %Y %H:%M:%S UTC",gmt);
+		if( s == 0 || s >= 99) {
+			fprintf(stderr,"strftime is doing strange things...\n");
+			return RET_ERROR;
+		}
+	}
 
 	if( distribution->signwith != NULL )
 		r = signature_startsignedfile(release->dirofdist, "Release",
@@ -1211,22 +1226,30 @@ retvalue release_prepare(struct release *release, struct distribution *distribut
 	}
 	writestring("\nDate: ");
 	writestring(buffer);
+	if( distribution->validfor > 0 ) {
+		writestring("\nValid-Until: ");
+		writestring(untilbuffer);
+	}
 	writestring("\nArchitectures:");
 	for( i = 0 ; i < distribution->architectures.count ; i++ ) {
+		architecture_t a = distribution->architectures.atoms[i];
+
 		/* Debian's topmost Release files do not list it, so we won't either */
-		if( strcmp(distribution->architectures.values[i],"source") == 0 )
+		if( a == architecture_source )
 			continue;
 		writechar(' ');
-		writestring(distribution->architectures.values[i]);
+		writestring(atoms_architectures[a]);
 	}
 	writestring("\nComponents:");
 	for( i = 0 ; i < distribution->components.count ; i++ ) {
+		component_t c = distribution->components.atoms[i];
+
 		writechar(' ');
 		if( release->fakecomponentprefix != NULL ) {
 			writestring(release->fakecomponentprefix);
 			writechar('/');
 		}
-		writestring(distribution->components.values[i]);
+		writestring(atoms_components[c]);
 	}
 	if( distribution->description != NULL ) {
 		writestring("\nDescription: ");
