@@ -26,6 +26,7 @@
 #include <malloc.h>
 #include <fcntl.h>
 #ifdef HAVE_LIBGPGME
+#include <gpg-error.h>
 #include <gpgme.h>
 #endif
 #include "globals.h"
@@ -39,16 +40,14 @@
 #include "readtextfile.h"
 #include "signature.h"
 
-extern int verbose;
-
 #ifdef HAVE_LIBGPGME
 static gpgme_ctx_t context = NULL;
 
-static retvalue gpgerror(gpgme_error_t err) {
+static retvalue gpgerror(gpg_error_t err) {
 	if( err != 0 ) {
 		fprintf(stderr,"gpgme gave %s error: %s\n",
-				gpgme_strsource(err), gpgme_strerror(err));
-		if( gpgme_err_code(err) == GPG_ERR_ENOMEM )
+				gpg_strsource(err), gpg_strerror(err));
+		if( gpg_err_code(err) == GPG_ERR_ENOMEM )
 			return RET_ERROR_OOM;
 		else
 			return RET_ERROR_GPGME;
@@ -57,7 +56,7 @@ static retvalue gpgerror(gpgme_error_t err) {
 }
 
 /* Quick&dirty passphrase asking */
-static gpgme_error_t signature_getpassphrase(UNUSED(void *hook), const char *uid_hint, UNUSED(const char *info), int prev_was_bad, int fd) {
+static gpg_error_t signature_getpassphrase(UNUSED(void *hook), const char *uid_hint, UNUSED(const char *info), int prev_was_bad, int fd) {
 	char *msg;
 	const char *p;
 
@@ -65,7 +64,7 @@ static gpgme_error_t signature_getpassphrase(UNUSED(void *hook), const char *uid
 			(uid_hint!=NULL)?uid_hint:"key",
 			(prev_was_bad!=0)?" again":"");
 	if( msg == NULL )
-		return gpgme_err_make(GPG_ERR_SOURCE_USER_1, GPG_ERR_ENOMEM);
+		return gpg_err_make(GPG_ERR_SOURCE_USER_1, GPG_ERR_ENOMEM);
 	p = getpass(msg);
 	write(fd, p, strlen(p));
 	write(fd, "\n", 1);
@@ -76,7 +75,7 @@ static gpgme_error_t signature_getpassphrase(UNUSED(void *hook), const char *uid
 
 retvalue signature_init(bool allowpassphrase){
 #ifdef HAVE_LIBGPGME
-	gpgme_error_t err;
+	gpg_error_t err;
 
 	if( context != NULL )
 		return RET_NOTHING;
@@ -117,7 +116,7 @@ static inline retvalue containskey(const char *key, const char *fingerprint) {
 		while( *keypart != '\0' && xisspace(*keypart) )
 			keypart++;
 		if( *keypart == '\0' )
-			/* nothing more to check, so nothing fullfilled */
+			/* nothing more to check, so nothing fulfilled */
 			return RET_NOTHING;
 		p = keypart;
 		while( *p != '\0' && !xisspace(*p) && *p != '|' )
@@ -144,16 +143,20 @@ static inline retvalue containskey(const char *key, const char *fingerprint) {
 }
 #endif /* HAVE_LIBGPGME */
 
-retvalue signature_check(const char *options, const char *releasegpg, const char *release) {
+retvalue signature_check(const struct strlist *requirements, const char *releasegpg, const char *release) {
 	retvalue r;
 #ifdef HAVE_LIBGPGME
-	gpgme_error_t err;
+	gpg_error_t err;
 	int fd,gpgfd;
 	gpgme_data_t dh,dh_gpg;
 	gpgme_verify_result_t result;
 	gpgme_signature_t s;
+	bool totalfulfilled[requirements->count], keyfulfills[requirements->count];
+	int i,j;
 #endif /* HAVE_LIBGPGME */
 
+	assert( requirements->count > 0 );
+
 	if( release == NULL || releasegpg == NULL )
 		return RET_ERROR_OOM;
 
@@ -162,6 +165,8 @@ retvalue signature_check(const char *options, const char *releasegpg, const char
 		return r;
 
 #ifdef HAVE_LIBGPGME
+	for( i = 0 ; i < requirements->count ; i++ )
+		totalfulfilled[i] = false;
 	/* Read the file and its signature into memory: */
 	gpgfd = open(releasegpg, O_RDONLY|O_NOCTTY);
 	if( gpgfd < 0 ) {
@@ -203,11 +208,20 @@ retvalue signature_check(const char *options, const char *releasegpg, const char
 		return RET_ERROR_GPGME;
 	}
 	for( s = result->signatures ; s != NULL ; s = s->next ) {
-		r = containskey(options, s->fpr);
-		if( RET_WAS_ERROR(r) )
-			return r;
-		if( r == RET_NOTHING ) {
-			if( gpgme_err_code(s->status) == GPG_ERR_NO_ERROR &&
+		bool keyused = false;
+
+		for( i = 0 ; i < requirements->count ; i++ ) {
+			r = containskey(requirements->values[i], s->fpr);
+			if( RET_WAS_ERROR(r) )
+				return r;
+			if( RET_IS_OK(r) ) {
+				keyused = true;
+				keyfulfills[i] = true;
+			} else
+				keyfulfills[i] = false;
+		}
+		if( !keyused ) {
+			if( gpg_err_code(s->status) == GPG_ERR_NO_ERROR &&
 					verbose > 10 ) {
 				printf("Valid signature with key '%s', but that key is not looked at.\n", s->fpr);
 			}
@@ -215,7 +229,7 @@ retvalue signature_check(const char *options, const char *releasegpg, const char
 			continue;
 		}
 		assert( RET_IS_OK(r) );
-		switch( gpgme_err_code(s->status) ) {
+		switch( gpg_err_code(s->status) ) {
 			case GPG_ERR_NO_ERROR:
 				return RET_OK;
 			case GPG_ERR_KEY_EXPIRED:
@@ -223,16 +237,22 @@ retvalue signature_check(const char *options, const char *releasegpg, const char
 "WARNING: valid signature in '%s' with '%s', which has been expired.\n"
 "         as the key was manually specified it is still accepted!\n",
 						releasegpg, s->fpr);
-				return RET_OK;
+				for( i = 0 ; i < requirements->count ; i++ ) {
+					if( keyfulfills[i] )
+						totalfulfilled[i] = true;
+				}
+				continue;
 			case GPG_ERR_CERT_REVOKED:
-				if( verbose >= 0 ) {
-					fprintf(stderr,
+				fprintf(stderr,
 "WARNING\n"
 "WARNING: valid signature in '%s' with '%s', which has been revoked.\n"
 "WARNING: as the key was manually specified it is still accepted!\n"
 "WARNING\n", releasegpg, s->fpr);
+				for( i = 0 ; i < requirements->count ; i++ ) {
+					if( keyfulfills[i] )
+						totalfulfilled[i] = true;
 				}
-				return RET_OK;
+				continue;
 			case GPG_ERR_SIG_EXPIRED:
 				if( verbose > 0 ) {
 					time_t timestamp = s->timestamp,
@@ -277,10 +297,30 @@ retvalue signature_check(const char *options, const char *releasegpg, const char
 		fprintf(stderr,
 "Error checking signature (gpgme returned unexpected value %d)!\n"
 "Please file a bug report, so reprepro can handle this in the future.\n",
-			gpgme_err_code(s->status));
+			gpg_err_code(s->status));
 		return RET_ERROR_GPGME;
 	}
-	return RET_NOTHING;
+	for( i = 0 ; i < requirements->count ; i++ ) {
+		if( totalfulfilled[i] )
+			continue;
+		j = 0;
+		while( j < requirements->count && !totalfulfilled[j] )
+			j++;
+		if( j >= requirements->count ) {
+			/* none of the requirements matches */
+			fprintf(stderr,
+"Error: No accepted signature for '%s' found in '%s'!\n",
+					release, releasegpg);
+		} else {
+			fprintf(stderr,
+"Error: '%s' misses requirement '%s'.\n"
+"At least one other requirement on that file was not missed,\n"
+"but all requirements have to be fulfilled to accept it.\n",
+					releasegpg, requirements->values[i]);
+		}
+		return RET_ERROR_BADSIG;
+	}
+	return RET_OK;
 #else /* HAVE_LIBGPGME */
 	fprintf(stderr,
 "ERROR: Cannot check signature as this reprepro binary is compiled with support\n"
@@ -293,7 +333,7 @@ retvalue signature_sign(const char *options, const char *filename, const char *s
 	retvalue r;
 	int ret, e;
 #ifdef HAVE_LIBGPGME
-	gpgme_error_t err;
+	gpg_error_t err;
 	gpgme_data_t dh,dh_gpg;
 #endif /* HAVE_LIBGPGME */
 
@@ -328,10 +368,10 @@ retvalue signature_sign(const char *options, const char *filename, const char *s
 		gpgme_signers_clear(context);
 /*		does not work:
  		err = gpgme_get_key(context, options, &key, 1);
-		if( gpgme_err_code(err) == GPG_ERR_AMBIGUOUS_NAME ) {
+		if( gpg_err_code(err) == GPG_ERR_AMBIGUOUS_NAME ) {
 			fprintf(stderr, "'%s' is too ambiguous for gpgme!\n", options);
 			return RET_ERROR;
-		} else if( gpgme_err_code(err) == GPG_ERR_INV_VALUE ) {
+		} else if( gpg_err_code(err) == GPG_ERR_INV_VALUE ) {
 			fprintf(stderr, "gpgme says '%s' is \"not a fingerprint or key ID\"!\n\n", options);
 			return RET_ERROR;
 		}
@@ -346,7 +386,7 @@ retvalue signature_sign(const char *options, const char *filename, const char *s
 		if( err != 0 )
 			return gpgerror(err);
 		err = gpgme_op_keylist_next(context, &key);
-		if( gpgme_err_code(err) == GPG_ERR_EOF ) {
+		if( gpg_err_code(err) == GPG_ERR_EOF ) {
 			fprintf(stderr,"Could not find any key matching '%s'!\n",options);
 			return RET_ERROR;
 		}
@@ -423,7 +463,7 @@ retvalue signature_sign(const char *options, const char *filename, const char *s
 		while( signature_len > 0 ) {
 			written = write(fd,p,signature_len);
 			if( written < 0 ) {
-				int e = errno;
+				e = errno;
 				fprintf(stderr, "Error %d writing to %s: %s\n",
 						e, signaturename,
 						strerror(e));
@@ -440,7 +480,7 @@ retvalue signature_sign(const char *options, const char *filename, const char *s
 #endif
 		ret = close(fd);
 		if( ret < 0 ) {
-			int e = errno;
+			e = errno;
 			fprintf(stderr, "Error %d writing to %s: %s\n",
 					e, signaturename,
 					strerror(e));
@@ -481,7 +521,7 @@ static retvalue checksigs(const char *filename, struct strlist *valid, struct st
 			if( RET_WAS_ERROR(r) )
 				return r;
 		}
-		switch( gpgme_err_code(s->status) ) {
+		switch( gpg_err_code(s->status) ) {
 			case GPG_ERR_NO_ERROR:
 				had_valid = true;
 				if( valid != NULL ) {
@@ -548,7 +588,7 @@ static retvalue checksigs(const char *filename, struct strlist *valid, struct st
 		fprintf(stderr,
 "Error checking signature (gpgme returned unexpected value %d)!\n"
 "Please file a bug report, so reprepro can handle this in the future.\n",
-			gpgme_err_code(s->status));
+			gpg_err_code(s->status));
 		return RET_ERROR_GPGME;
 	}
 	if( broken != NULL && had_broken && ! had_valid )
@@ -595,7 +635,7 @@ static inline void extractchunk(const char *buffer, const char **begin, const ch
 static retvalue extract_signed_data(const char *buffer, size_t bufferlen, const char *filenametoshow, char **chunkread, /*@null@*/ /*@out@*/struct strlist *validkeys, /*@null@*/ /*@out@*/ struct strlist *allkeys, bool *brokensignature, bool *failed) {
 	const char *startofchanges,*endofchanges,*afterchanges;
 	char *chunk;
-	gpgme_error_t err;
+	gpg_error_t err;
 	gpgme_data_t dh,dh_gpg;
 	size_t plain_len;
 	char *plain_data;
@@ -620,8 +660,8 @@ static retvalue extract_signed_data(const char *buffer, size_t bufferlen, const
 		return gpgerror(err);
 	}
 	err = gpgme_op_verify(context, dh_gpg, NULL, dh);
-	if( gpgme_err_code(err) == GPG_ERR_NO_DATA ) {
-		if( verbose > -1 )
+	if( gpg_err_code(err) == GPG_ERR_NO_DATA ) {
+		if( verbose > 5 )
 			fprintf(stderr,"Data seems not to be signed trying to use directly....\n");
 		gpgme_data_release(dh);
 		gpgme_data_release(dh_gpg);
@@ -759,7 +799,7 @@ retvalue signature_readsignedchunk(const char *filename, const char *filenametos
 
 	/* fast-track unsigned chunks: */
 	if( startofchanges[0] != '-' && *afterchanges == '\0' ) {
-		if( verbose > -1 )
+		if( verbose > 5 )
 			fprintf(stderr,"Data seems not to be signed trying to use directly...\n");
 		len = chunk_extract(chunk, chunk, &afterchunk);
 		assert( *afterchunk == '\0' );
@@ -871,14 +911,14 @@ struct signedfile {
 };
 
 
-static retvalue newpossiblysignedfile(const char *directory, const char *basename, struct signedfile **out) {
+static retvalue newpossiblysignedfile(const char *directory, const char *basefilename, struct signedfile **out) {
 	struct signedfile *n;
 
 	n = calloc(1, sizeof(struct signedfile));
 	if( n == NULL )
 		return RET_ERROR_OOM;
 	n->fd = -1;
-	n->plainfilename = calc_dirconcat(directory, basename);
+	n->plainfilename = calc_dirconcat(directory, basefilename);
 	if( n->plainfilename == NULL ) {
 		free(n);
 		return RET_ERROR_OOM;
@@ -926,11 +966,11 @@ void signedfile_free(struct signedfile *f) {
 	return;
 }
 
-retvalue signature_startsignedfile(const char *directory, const char *basename, UNUSED(const char *options), struct signedfile **out) {
+retvalue signature_startsignedfile(const char *directory, const char *basefilename, UNUSED(const char *options), struct signedfile **out) {
 	retvalue r;
 	struct signedfile *n;
 
-	r = newpossiblysignedfile(directory, basename, &n);
+	r = newpossiblysignedfile(directory, basefilename, &n);
 	if( RET_WAS_ERROR(r) )
 		return r;
 	// create object to place data into...
@@ -938,11 +978,11 @@ retvalue signature_startsignedfile(const char *directory, const char *basename,
 	return RET_OK;
 }
 
-retvalue signature_startunsignedfile(const char *directory, const char *basename, struct signedfile **out) {
+retvalue signature_startunsignedfile(const char *directory, const char *basefilename, struct signedfile **out) {
 	retvalue r;
 	struct signedfile *n;
 
-	r = newpossiblysignedfile(directory, basename, &n);
+	r = newpossiblysignedfile(directory, basefilename, &n);
 	if( RET_WAS_ERROR(r) )
 		return r;
 	*out = n;
