@@ -1,5 +1,5 @@
 /*  This file is part of "reprepro"
- *  Copyright (C) 2003,2004,2005,2006,2007,2008 Bernhard R. Link
+ *  Copyright (C) 2003,2004,2005,2006,2007,2008,2009 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
  *  published by the Free Software Foundation.
@@ -35,8 +35,6 @@
 #include "tracking.h"
 #include "signature.h"
 
-extern int verbose;
-
 /* split a "<md5> <size> <filename>" into md5sum and filename */
 static retvalue calc_parsefileline(const char *fileline, /*@out@*/char **filename) {
 	const char *p, *fn, *fnend;
@@ -94,14 +92,19 @@ static retvalue getBasenames(const struct strlist *filelines,/*@out@*/struct str
 		return r;
 	r = RET_NOTHING;
 	for( i = 0 ; i < filelines->count ; i++ ) {
-		char *basename IFSTUPIDCC(=NULL);
-		const char *fileline=filelines->values[i];
+		char *basefilename IFSTUPIDCC(=NULL);
+		const char *fileline = filelines->values[i];
 
-		r = calc_parsefileline(fileline, &basename);
+		r = calc_parsefileline(fileline, &basefilename);
+		if( r == RET_NOTHING ) {
+			fprintf(stderr, "Malformed Files: line '%s'!\n",
+					fileline);
+			r = RET_ERROR;
+		}
 		if( RET_WAS_ERROR(r) )
 			break;
 
-		r = strlist_add(basenames,basename);
+		r = strlist_add(basenames, basefilename);
 		if( RET_WAS_ERROR(r) ) {
 			break;
 		}
@@ -115,18 +118,6 @@ static retvalue getBasenames(const struct strlist *filelines,/*@out@*/struct str
 	return r;
 }
 
-retvalue sources_getname(const char *control, char **packagename){
-	retvalue r;
-
-	r = chunk_getvalue(control,"Package",packagename);
-	if( RET_WAS_ERROR(r) )
-		return r;
-	if( r == RET_NOTHING ) {
-		fprintf(stderr, "Missing 'Package' field in chunk:'%s'\n", control);
-		return RET_ERROR;
-	}
-	return r;
-}
 retvalue sources_getversion(const char *control, char **version) {
 	retvalue r;
 
@@ -140,7 +131,12 @@ retvalue sources_getversion(const char *control, char **version) {
 	return r;
 }
 
-retvalue sources_getinstalldata(const struct target *t, const char *packagename, UNUSED(const char *version), const char *chunk, char **control, struct strlist *filekeys, struct checksumsarray *origfiles, /*@null@*//*@out@*/enum filetype *type_p) {
+retvalue sources_getarchitecture(UNUSED(const char *chunk), architecture_t *architecture_p) {
+	*architecture_p = architecture_source;
+	return RET_OK;
+}
+
+retvalue sources_getinstalldata(const struct target *t, const char *packagename, UNUSED(const char *version), architecture_t architecture, const char *chunk, char **control, struct strlist *filekeys, struct checksumsarray *origfiles) {
 	retvalue r;
 	char *origdirectory, *directory, *mychunk;
 	struct strlist myfilekeys;
@@ -148,6 +144,8 @@ retvalue sources_getinstalldata(const struct target *t, const char *packagename,
 	struct checksumsarray files;
 	enum checksumtype cs;
 
+	assert( architecture == architecture_source );
+
 	for( cs = cs_md5sum ; cs < cs_hashCOUNT ; cs++ ) {
 		assert( source_checksum_names[cs] != NULL );
 		r = chunk_getextralinelist(chunk, source_checksum_names[cs],
@@ -176,8 +174,13 @@ retvalue sources_getinstalldata(const struct target *t, const char *packagename,
 
 	r = chunk_getvalue(chunk, "Directory", &origdirectory);
 	if( r == RET_NOTHING ) {
+/* Flat repositories can come without this, TODO: add warnings in other cases
 		fprintf(stderr,"Missing 'Directory' entry in '%s'!\n",chunk);
 		r = RET_ERROR;
+*/
+		origdirectory = strdup(".");
+		if( FAILEDTOALLOC(origdirectory) )
+			r = RET_ERROR_OOM;
 	}
 	if( RET_WAS_ERROR(r) ) {
 		checksumsarray_done(&files);
@@ -195,7 +198,7 @@ retvalue sources_getinstalldata(const struct target *t, const char *packagename,
 		return r;
 	}
 
-	directory = calc_sourcedir(t->component, packagename);
+	directory = calc_sourcedir(t->component_atom, packagename);
 	if( directory == NULL )
 		r = RET_ERROR_OOM;
 	else
@@ -209,7 +212,8 @@ retvalue sources_getinstalldata(const struct target *t, const char *packagename,
 	r = calc_inplacedirconcats(origdirectory, &files.names);
 	free(origdirectory);
 	if( !RET_WAS_ERROR(r) ) {
-		mychunk = chunk_replacefield(chunk, "Directory", directory);
+		mychunk = chunk_replacefield(chunk,
+				"Directory", directory, true);
 		if( mychunk == NULL )
 			r = RET_ERROR_OOM;
 	}
@@ -222,8 +226,6 @@ retvalue sources_getinstalldata(const struct target *t, const char *packagename,
 	*control = mychunk;
 	strlist_move(filekeys, &myfilekeys);
 	checksumsarray_move(origfiles, &files);
-	if( type_p != NULL )
-		*type_p = ft_SOURCE;
 	return RET_OK;
 }
 
@@ -320,14 +322,11 @@ retvalue sources_getchecksums(const char *chunk, struct checksumsarray *out) {
 	return RET_OK;
 }
 
-char *sources_getupstreamindex(const char *suite_from, const char *component_from, UNUSED(const char *architecture)) {
-	return mprintf("dists/%s/%s/source/Sources.gz",suite_from,component_from);
-}
-
 retvalue sources_doreoverride(const struct distribution *distribution,const char *packagename,const char *controlchunk,/*@out@*/char **newcontrolchunk) {
 	const struct overrideinfo *o;
 	struct fieldtoadd *fields;
 	char *newchunk;
+	retvalue r;
 
 	if( interrupted() )
 		return RET_ERROR_INTERRUPTED;
@@ -336,10 +335,11 @@ retvalue sources_doreoverride(const struct distribution *distribution,const char
 	if( o == NULL )
 		return RET_NOTHING;
 
-	fields = override_addreplacefields(o,NULL);
-	if( fields == NULL )
-		return RET_ERROR_OOM;
-	newchunk = chunk_replacefields(controlchunk,fields,"Files");
+	r = override_allreplacefields(o, &fields);
+	if( RET_WAS_ERROR(r) )
+		return r;
+	newchunk = chunk_replacefields(controlchunk, fields,
+			"Directory", true);
 	addfield_free(fields);
 	if( newchunk == NULL )
 		return RET_ERROR_OOM;
@@ -561,7 +561,7 @@ retvalue sources_complete(const struct dsc_headers *dsc, const char *directory,
 	name = deletefield_new("Source",name);
 	if( name == NULL )
 		return RET_ERROR_OOM;
-	newchunk2  = chunk_replacefields(dsc->control,name,"Format");
+	newchunk2  = chunk_replacefields(dsc->control, name, "Format", true);
 	addfield_free(name);
 	if( newchunk2 == NULL )
 		return RET_ERROR_OOM;
@@ -596,7 +596,7 @@ retvalue sources_complete(const struct dsc_headers *dsc, const char *directory,
 		return RET_ERROR_OOM;
 	}
 
-	newchunk  = chunk_replacefields(newchunk2,replace,"Files");
+	newchunk  = chunk_replacefields(newchunk2, replace, "Files", false);
 	free(newsha256lines);
 	free(newsha1lines);
 	free(newfilelines);
@@ -611,3 +611,58 @@ retvalue sources_complete(const struct dsc_headers *dsc, const char *directory,
 	return RET_OK;
 }
 
+char *calc_source_basename(const char *name, const char *version) {
+	const char *v = strchr(version, ':');
+	if( v != NULL )
+		v++;
+	else
+		v = version;
+	return mprintf("%s_%s.dsc", name, v);
+}
+
+char *calc_sourcedir(component_t component, const char *sourcename) {
+
+	assert( *sourcename != '\0' );
+
+	if( sourcename[0] == 'l' && sourcename[1] == 'i' &&
+			sourcename[2] == 'b' && sourcename[3] != '\0' )
+		return mprintf("pool/%s/lib%c/%s",
+				atoms_components[component],
+				sourcename[3], sourcename);
+	else if( *sourcename != '\0' )
+		return mprintf("pool/%s/%c/%s",
+				atoms_components[component],
+				sourcename[0], sourcename);
+	else
+		return NULL;
+}
+
+char *calc_filekey(component_t component, const char *sourcename, const char *filename) {
+	if( sourcename[0] == 'l' && sourcename[1] == 'i' &&
+			sourcename[2] == 'b' && sourcename[3] != '\0' )
+		return mprintf("pool/%s/lib%c/%s/%s",
+				atoms_components[component],
+				sourcename[3], sourcename, filename);
+	else if( *sourcename != '\0' )
+		return mprintf("pool/%s/%c/%s/%s",
+				atoms_components[component],
+				sourcename[0], sourcename, filename);
+	else
+		return NULL;
+}
+
+char *calc_byhanddir(component_t component, const char *sourcename, const char *version) {
+	if( sourcename[0] == 'l' && sourcename[1] == 'i' &&
+			sourcename[2] == 'b' && sourcename[3] != '\0' )
+		return mprintf("pool/%s/lib%c/%s/%s_%s_byhand",
+				atoms_components[component],
+				sourcename[3], sourcename,
+				sourcename, version);
+	else if( *sourcename != '\0' )
+		return mprintf("pool/%s/%c/%s/%s_%s_byhand",
+				atoms_components[component],
+				sourcename[0], sourcename,
+				sourcename, version);
+	else
+		return NULL;
+}
