@@ -7,6 +7,9 @@
 #ifndef REPREPRO_NAMES_H
 #include "names.h"
 #endif
+#ifndef REPREPRO_ATOMS_H
+#include "atoms.h"
+#endif
 #ifndef REPREPRO_DATABASE_H
 #include "database.h"
 #endif
@@ -23,35 +26,33 @@
 struct target;
 struct alloverrides;
 
-typedef retvalue get_name(const char *, /*@out@*/char **);
 typedef retvalue get_version(const char *, /*@out@*/char **);
-typedef retvalue get_installdata(const struct target *,const char *,const char *,const char *,/*@out@*/char **,/*@out@*/struct strlist *,/*@out@*/struct checksumsarray *,/*@null@*//*@out@*/enum filetype *);
+typedef retvalue get_architecture(const char *, /*@out@*/architecture_t *);
+typedef retvalue get_installdata(const struct target *, const char *, const char *, architecture_t, const char *, /*@out@*/char **, /*@out@*/struct strlist *, /*@out@*/struct checksumsarray *);
 /* md5sums may be NULL */
 typedef retvalue get_filekeys(const char *, /*@out@*/struct strlist *);
 typedef retvalue get_checksums(const char *, /*@out@*/struct checksumsarray *);
-typedef char *get_upstreamindex(const char *suite_from, const char *component_from, const char *architecture);
 typedef retvalue do_reoverride(const struct distribution *,const char *packagename,const char *controlchunk,/*@out@*/char **newcontrolchunk);
 typedef retvalue do_retrack(const char *packagename, const char *controlchunk, trackingdb, struct database *);
 typedef retvalue get_sourceandversion(const char *chunk, const char *packagename, /*@out@*/char **source, /*@out@*/char **version);
 
 struct target {
 	char *codename;
-	char *component;
-	char *architecture;
+	component_t component_atom;
+	architecture_t architecture_atom;
+	packagetype_t packagetype_atom;
 	char *identifier;
-	/* "deb" "udeb" or "dsc" */
-	/*@observer@*/const char *packagetype;
 	/* links into the correct description in distribution */
 	/*@dependent@*/const struct exportmode *exportmode;
 	/* the directory relative to <distdir>/<codename>/ to use */
 	char *relativedirectory;
 	/* functions to use on the packages included */
-	get_name *getname;
 	get_version *getversion;
+	/* binary packages might be "all" or the architecture of the target */
+	get_architecture *getarchitecture;
 	get_installdata *getinstalldata;
 	get_filekeys *getfilekeys;
 	get_checksums *getchecksums;
-	get_upstreamindex *getupstreamindex;
 	get_sourceandversion *getsourceandversion;
 	do_reoverride *doreoverride;
 	do_retrack *doretrack;
@@ -62,11 +63,13 @@ struct target {
 	struct target *next;
 	/* is initialized as soon as needed: */
 	struct table *packages;
+	/* do not allow write operations */
+	bool readonly;
 };
 
-retvalue target_initialize_ubinary(const char *codename,const char *component,const char *architecture,/*@dependent@*/const struct exportmode *exportmode,/*@out@*/struct target **target);
-retvalue target_initialize_binary(const char *codename,const char *component,const char *architecture,/*@dependent@*/const struct exportmode *exportmode,/*@out@*/struct target **target);
-retvalue target_initialize_source(const char *codename,const char *component,/*@dependent@*/const struct exportmode *exportmode,/*@out@*/struct target **target);
+retvalue target_initialize_ubinary(const char *codename, component_t, architecture_t, /*@dependent@*/const struct exportmode *, bool readonly, /*@out@*/struct target **);
+retvalue target_initialize_binary(const char *codename, component_t, architecture_t, /*@dependent@*/const struct exportmode *, bool readonly, /*@out@*/struct target **);
+retvalue target_initialize_source(const char *codename, component_t, /*@dependent@*/const struct exportmode *, bool readonly, /*@out@*/struct target **);
 retvalue target_free(struct target *target);
 
 retvalue target_export(struct target *target, struct database *, bool onlyneeded, bool snapshot, struct release *release);
@@ -138,13 +141,13 @@ static inline retvalue target_closeiterator(struct target_cursor *tc) {
 
 /* The following calls can only be called if target_initpackagesdb was called before: */
 struct logger;
-retvalue target_addpackage(struct target *, /*@null@*/struct logger *, struct database *, const char *name, const char *version, const char *control, const struct strlist *filekeys, /*@null@*/ bool *usedmarker, bool downgrade, /*@null@*/struct strlist *dereferencedfilekeys, /*@null@*/struct trackingdata *, enum filetype);
+retvalue target_addpackage(struct target *, /*@null@*/struct logger *, struct database *, const char *name, const char *version, const char *control, const struct strlist *filekeys, bool downgrade, /*@null@*/struct trackingdata *, enum filetype);
 retvalue target_checkaddpackage(struct target *target, const char *name, const char *version, bool tracking, bool permitnewerold);
-retvalue target_removepackage(struct target *, /*@null@*/struct logger *, struct database *, const char *name, /*@null@*/struct strlist *dereferencedfilekeys, struct trackingdata *);
+retvalue target_removepackage(struct target *, /*@null@*/struct logger *, struct database *, const char *name, struct trackingdata *);
 /* like target_removepackage, but do not read control data yourself but use available */
-retvalue target_removereadpackage(struct target *, /*@null@*/struct logger *, struct database *, const char *name, const char *oldcontrol, struct strlist *, /*@null@*/struct trackingdata *);
+retvalue target_removereadpackage(struct target *, /*@null@*/struct logger *, struct database *, const char *name, const char *oldcontrol, /*@null@*/struct trackingdata *);
 /* Like target_removepackage, but delete the package record by cursor */
-retvalue target_removepackage_by_cursor(struct target_cursor *, /*@null@*/struct logger *, struct database *, struct strlist *, /*@null@*/struct trackingdata *);
+retvalue target_removepackage_by_cursor(struct target_cursor *, /*@null@*/struct logger *, struct database *, /*@null@*/struct trackingdata *);
 
 retvalue package_check(struct database *, struct distribution *, struct target *, const char *, const char *, void *);
 retvalue target_rereference(struct target *, struct database *);
@@ -153,12 +156,12 @@ retvalue target_reoverride(struct target *, struct distribution *, struct databa
 
 retvalue package_rerunnotifiers(struct database *, struct distribution *, struct target *, const char *, const char *, void *);
 
-static inline bool target_matches(const struct target *t, const char *component, const char *architecture, const char *packagetype) {
-	if( component != NULL && strcmp(component,t->component) != 0 )
+static inline bool target_matches(const struct target *t, component_t component, architecture_t architecture, packagetype_t packagetype) {
+	if( atom_defined(component) && component != t->component_atom )
 		return false;
-	if( architecture != NULL && strcmp(architecture,t->architecture) != 0 )
+	if( atom_defined(architecture) && architecture != t->architecture_atom )
 		return false;
-	if( packagetype != NULL && strcmp(packagetype,t->packagetype) != 0 )
+	if( atom_defined(packagetype) && packagetype != t->packagetype_atom )
 		return false;
 	return true;
 }
